/*
 *  File: EchoClient.java
 *
 *  This simple client applet may be used with EchoServer 
 *  or ThreadedEchoServer.  Note:  Since
 *  EchoClient is an applet, server and client must be 
 *  running on the same host.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 * 
 */
 
import java.awt.*;
import java.awt.event.*;
import java.net.Socket;
import java.io.*;

public class EchoClient extends java.applet.Applet
                        implements ActionListener, TextListener 
{ 
  final int defaultPort = 7776;
  String defaultPortStr;
  
  // networking and IO variables:
  int port;
  String host;
  Socket socket;
  BufferedReader in;
  PrintWriter out;
  
  // user interface components:
  TextField textfield;
  TextArea textarea;
  Button button;
  
  // dynamic button labels:
  String connectLabel = "   Connect   ";
  String disconnectLabel = "Disconnect";
  
  // prompt:
  String prompt = "> ";   // must not end with "\n" or "\r"
  
  public void init() 
  {
    // initialization:
    host = this.getCodeBase().getHost();
    defaultPortStr = ( new Integer( defaultPort ) ).toString();
    
    // user control panel:
    Panel controlPanel = new Panel();
    controlPanel.add( new Label( "Port:" ) );
    textfield = new TextField( defaultPortStr );
    controlPanel.add( textfield );
    button = new Button( connectLabel );
    controlPanel.add( button );
    
    // applet user interface:
    setLayout( new BorderLayout() );
    add( controlPanel, BorderLayout.NORTH );
    textarea = new TextArea( 8, 60 );
    add( textarea, BorderLayout.CENTER );
    
    button.addActionListener( this );
    textarea.addTextListener( this );
    
    // give button focus:
    button.requestFocus();
    
  } 

  public void actionPerformed( ActionEvent event ) 
  {
   if ( button.getLabel().equals( connectLabel ) ) 
    { openConnection();
    } 
    else 
    { closeConnection();
    }
    
  }
  
  public void textValueChanged( TextEvent event ) 
  {
    String text = textarea.getText();
    if ( text.endsWith( "\n" ) ) {
    
      // determine client output:
      int index = text.lastIndexOf( prompt ) + prompt.length();
      String output = text.substring( index ).trim();
      
      // send client output to server:
      out.println( output );
      
      try 
       { // update textarea:
         textarea.append( serverResponse() );
       } 
      catch ( IOException e ) 
       {
        System.err.println( e );
        textarea.setText( e.getMessage() );
        closeConnection();
       }
    }
    
  }

  private void openConnection() 
  {
    try 
    {
        // get the port number and open a socket:
        int port = Integer.parseInt( textfield.getText() );
        socket = new Socket( host, port );
        
        // open IO streams:
        in = new BufferedReader(
               new InputStreamReader( socket.getInputStream() ) );
        boolean autoflush;
        out =
          new PrintWriter( socket.getOutputStream(), autoflush = true );
        
        // initialize textarea:
        textarea.setText( serverResponse() );
        
        // toggle button label:
        button.setLabel( disconnectLabel );
        
       } 
      catch ( NumberFormatException nfe ) 
       {
        System.err.println( nfe );
        textfield.setText( defaultPortStr );
        textarea.setText( nfe.getMessage() );
        
       } 
      catch ( IOException e ) 
       {
        System.err.println( e );
        textarea.setText( e.getMessage() );
        closeConnection();
       }
  }
  
  private String serverResponse() throws IOException 
  {
    String text = "";
    String input = in.readLine();
    if ( input == null ) 
     {
      throw new IOException( "Socket closed unexpectedly" );
     } 
     else 
     {
      text += input + "\n";
     }
    return text + prompt;

  }
  
  private void closeConnection() 
  {
   try 
    {
      socket.close();
    } 
   catch ( IOException e ) 
    {
      System.err.println( e );
    }

    textarea.setText( "" );
    button.setLabel( connectLabel );
    button.requestFocus();
    
  }
  
}
