/* 
 *  File:  ServerDetector.java
 *
 *  This server-side application detects servers running on 
 *  the current host.  It does this by attempting to run a 
 *  server at each port.
 *
 *  Usage:    % java ServerDetector [<minPort> [<maxPort>]]
 *  Example:  % java ServerDetector 5000 5999 | more
 * 
 *  Note:  Public ports are in the range 1024--65535.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *
 */

import java.net.ServerSocket;
import java.io.IOException;

class ServerDetector {  

  // by default, examine the first few public ports:
  private static int minPort = 1024;
  private static int maxPort = minPort + 20;

  public static void main( String[] args ) {  

    checkArgs( args );
    
    ServerSocket serversocket;

    for ( int port = minPort; port <= maxPort; port++ ) {
      try {
        serversocket = new ServerSocket( port );
        System.out.println( "Port " + port + ":  available" );
        serversocket.close();
      } catch ( IOException e ) {
        System.out.println( "Port " + port + ":  " + e.getMessage() );
      }
    }

  } 
  
  private static void checkArgs( String[] args ) {
  
    // determine port range:
    if ( args.length > 0 ) {
      if ( args.length > 2 ) {
        System.err.println( "Wrong number of arguments: " + args.length );
        System.out.println( "Ignoring all command-line arguments" );
      } else {
        try {
          maxPort = minPort = Integer.parseInt( args[0] );
          if ( args.length == 2 ) {
            try {
              maxPort = Integer.parseInt( args[1] );
            } catch ( NumberFormatException e ) {
              System.err.println( e );
              System.out.println( "Bad argument: " + args[1] );
            }
          }
        } catch ( NumberFormatException e ) {
          System.err.println( e );
          System.out.println( "Bad argument: " + args[0] );
          System.out.println( "Ignoring all command-line arguments" );
        }
      }
    }

  }

}
