/*
 *  File:  SocketTest.java
 *
 *  This simple client opens a socket to a date server, gets the 
 *  date, and displays it on the command line.  This version of 
 *  SocketTest optionally takes a host name on the command line.
 *
 *  Usage:    % java SocketTest [<hostname>]
 *  Example:  % java SocketTest kopernik.npac.syr.edu
 *
 *  Adapted from an example of the same name by
 *  Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 *
 */

import java.io.*;
import java.net.Socket;

class SocketTest
{  public static void main( String[] args )
   {  try
      {  
         // default host:
         String hostname = "osprey7.npac.syr.edu";
         
         // determine the host:
         if ( args.length > 0 ) hostname = args[0];
         
         // open socket connection to 'date' daemon at port 13:
         Socket socket = new Socket( hostname, 13 );

         // open an input stream:
         BufferedReader in =
           new BufferedReader(
             new InputStreamReader( socket.getInputStream() ) );

         // read and echo lines of input:
         while ( true )
         {  String str = in.readLine();
            if ( str == null ) break;
            System.out.println( hostname + ": " + str );
         }
         
         // close stream and socket:
         in.close();
         socket.close();
      } 
      catch ( IOException e ) {
        System.err.println( e );
      }
   }
}
